@@ -32,7 +32,7 @@
 
 #include <config.h>
 
-RCSID("$Id: su.c 16576 2006-01-17 15:38:18Z lha $");
+RCSID("$Id: su.c 21179 2007-06-19 15:16:41Z lha $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -440,6 +440,9 @@ main(int argc, char **argv)
     else
 	su_user = argv[optind++];
 
+    if (!issuid() && getuid() != 0)
+	warnx("Not setuid and you are root, expect this to fail");
+
     pwd = k_getpwnam(su_user);
     if(pwd == NULL)
 	errx (1, "unknown login %s", su_user);
