@@ -54,7 +54,7 @@
 
 #include <config.h>
 
-RCSID("$Id: encrypt.c,v 1.17 1997/05/20 18:40:37 bg Exp $");
+RCSID("$Id: encrypt.c 16802 2006-03-23 19:36:31Z lha $");
 
 #if	defined(ENCRYPTION)
 
@@ -64,13 +64,14 @@ RCSID("$Id: encrypt.c,v 1.17 1997/05/20 18:40:37 bg Exp $");
 #include "encrypt.h"
 #include "misc.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <roken.h>
 #ifdef SOCKS
 #include <socks.h>
 #endif
 
-#include <roken.h>
 
 /*
  * These functions pointers point to the current routines
@@ -88,7 +89,7 @@ static int autoencrypt = 0;
 static int autodecrypt = 0;
 static int havesessionkey = 0;
 static int Server = 0;
-static char *Name = "Noname";
+static const char *Name = "Noname";
 
 #define	typemask(x)	((x) > 0 ? 1 << ((x)-1) : 0)
 
@@ -174,7 +175,7 @@ static struct key_info {
 };
 
 void
-encrypt_init(char *name, int server)
+encrypt_init(const char *name, int server)
 {
     Encryptions *ep = encryptions;
 
@@ -435,7 +436,7 @@ encrypt_send_support(void)
 	 */
 	if (!Server && autodecrypt)
 	    encrypt_send_request_start();
-	net_write(str_send, str_suplen);
+	telnet_net_write(str_send, str_suplen);
 	printsub('>', &str_send[2], str_suplen - 2);
 	str_suplen = 0;
     }
@@ -635,7 +636,7 @@ encrypt_reply(unsigned char *data, int cnt)
 }
 
 /*
- * Called when a ENCRYPT START command is received.
+ * Called when ENCRYPT START is received.
  */
 void
 encrypt_start(unsigned char *data, int cnt)
@@ -654,7 +655,7 @@ encrypt_start(unsigned char *data, int cnt)
 	return;
     }
 
-    if (ep = finddecryption(decrypt_mode)) {
+    if ((ep = finddecryption(decrypt_mode))) {
 	decrypt_input = ep->input;
 	if (encrypt_verbose)
 	    printf("[ Input is now decrypted with type %s ]\r\n",
@@ -728,7 +729,8 @@ encrypt_request_start(unsigned char *data, int cnt)
 
 static unsigned char str_keyid[(MAXKEYLEN*2)+5] = { IAC, SB, TELOPT_ENCRYPT };
 
-void encrypt_keyid(struct key_info *kp, unsigned char *keyid, int len)
+static void
+encrypt_keyid(struct key_info *kp, unsigned char *keyid, int len)
 {
     Encryptions *ep;
     int dir = kp->dir;
@@ -796,7 +798,7 @@ void encrypt_send_keyid(int dir, unsigned char *keyid, int keylen, int saveit)
     }
     *strp++ = IAC;
     *strp++ = SE;
-    net_write(str_keyid, strp - str_keyid);
+    telnet_net_write(str_keyid, strp - str_keyid);
     printsub('>', &str_keyid[2], strp - str_keyid - 2);
 }
 
@@ -855,7 +857,7 @@ encrypt_start_output(int type)
     }
     *p++ = IAC;
     *p++ = SE;
-    net_write(str_start, p - str_start);
+    telnet_net_write(str_start, p - str_start);
     net_encrypt();
     printsub('>', &str_start[2], p - &str_start[2]);
     /*
@@ -881,7 +883,7 @@ encrypt_send_end(void)
 	return;
 
     str_end[3] = ENCRYPT_END;
-    net_write(str_end, sizeof(str_end));
+    telnet_net_write(str_end, sizeof(str_end));
     net_encrypt();
     printsub('>', &str_end[2], sizeof(str_end) - 2);
     /*
@@ -909,7 +911,7 @@ encrypt_send_request_start(void)
     }
     *p++ = IAC;
     *p++ = SE;
-    net_write(str_start, p - str_start);
+    telnet_net_write(str_start, p - str_start);
     printsub('>', &str_start[2], p - &str_start[2]);
     if (encrypt_debug_mode)
 	printf(">>>%s: Request input to be encrypted\r\n", Name);
@@ -919,7 +921,7 @@ void
 encrypt_send_request_end(void)
 {
     str_end[3] = ENCRYPT_REQEND;
-    net_write(str_end, sizeof(str_end));
+    telnet_net_write(str_end, sizeof(str_end));
     printsub('>', &str_end[2], sizeof(str_end) - 2);
 
     if (encrypt_debug_mode)
@@ -938,6 +940,25 @@ void encrypt_wait(void)
 	    return;
 }
 
+int
+encrypt_delay(void)
+{
+    if(!havesessionkey ||
+       (I_SUPPORT_ENCRYPT & remote_supports_decrypt) == 0 ||
+       (I_SUPPORT_DECRYPT & remote_supports_encrypt) == 0)
+	return 0;
+    if(!(encrypt_output && decrypt_input))
+	return 1;
+    return 0;
+}
+
+int encrypt_is_encrypting()
+{
+    if (encrypt_output && decrypt_input)
+	return 1;
+    return 0;
+}
+
 void
 encrypt_debug(int mode)
 {
