@@ -33,7 +33,7 @@
 
 #include "telnetd.h"
 
-RCSID("$Id: telnetd.c,v 1.75.2.2 2005/12/01 15:29:02 lha Exp $");
+RCSID("$Id: telnetd.c 18742 2006-10-21 13:37:28Z lha $");
 
 #ifdef _SC_CRAY_SECURE_SYS
 #include <sys/sysv.h>
@@ -59,19 +59,12 @@ int	auth_level = 0;
 
 extern	int utmp_len;
 int	registerd_host_only = 0;
-
-#undef NOERROR
-
-#ifdef	STREAMSPTY
-# include <stropts.h>
-# include <termios.h>
-#ifdef HAVE_SYS_UIO_H
-#include <sys/uio.h>
-#endif /* HAVE_SYS_UIO_H */
-#ifdef HAVE_SYS_STREAM_H
-#include <sys/stream.h>
+#ifdef ENCRYPTION
+int	require_encryption = 0;
 #endif
 
+#ifdef STREAMSPTY
+
 #ifdef _AIX
 #include <sys/termio.h>
 #endif
@@ -137,6 +130,9 @@ char valid_opts[] = "Bd:hklnS:u:UL:y"
 #ifdef AUTHENTICATION
 		    "a:X:z"
 #endif
+#ifdef ENCRYPTION
+                     "e"
+#endif
 #ifdef DIAGNOSTICS
 		    "D:"
 #endif
@@ -248,6 +244,11 @@ main(int argc, char **argv)
 	    break;
 #endif /* DIAGNOSTICS */
 
+#ifdef ENCRYPTION
+	case 'e':
+	    require_encryption = 1;
+	    break;
+#endif
 
 	case 'h':
 	    hostinfo = 0;
@@ -549,6 +550,15 @@ getterminaltype(char *name, size_t name_sz)
     if (his_state_is_will(TELOPT_ENCRYPT)) {
 	encrypt_wait();
     }
+    if (require_encryption) {
+
+	while (encrypt_delay())
+	    if (telnet_spin())
+		fatal(net, "Failed while waiting for encryption");
+
+	if (!encrypt_is_encrypting())
+	    fatal(net, "Encryption required but not turned on by client");
+    }
 #endif
     if (his_state_is_will(TELOPT_TSPEED)) {
 	static unsigned char sb[] =
@@ -790,12 +800,12 @@ show_issue(void)
 	    size_t len = strcspn(buf, "\r\n");
 	    if(len == strlen(buf)) {
 		/* there's no newline */
-		writenet((unsigned char*)buf, len);
+		writenet(buf, len);
 	    } else {
 		/* replace newline with \r\n */
 		buf[len] = '\0';
-		writenet((unsigned char*)buf, len);
-		writenet((unsigned char*)"\r\n", 2);
+		writenet(buf, len);
+		writenet("\r\n", 2);
 	    }
 	}
 	fclose(f);
