@@ -8,7 +8,7 @@
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: xnlock.c,v 1.102 2005/04/06 11:21:59 lha Exp $");
+RCSID("$Id: xnlock.c 18745 2006-10-21 14:09:56Z lha $");
 #endif
 
 #include <stdio.h>
@@ -19,10 +19,6 @@ RCSID("$Id: xnlock.c,v 1.102 2005/04/06 11:21:59 lha Exp $");
 #include <X11/Intrinsic.h>
 #include <X11/keysym.h>
 #include <X11/Shell.h>
-#include <X11/Xos.h>
-#ifdef strerror
-#undef strerror
-#endif
 #include <ctype.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -244,9 +240,10 @@ init_words (int argc, char **argv)
 		appres.file = argv[i];
 		i++;
 	    } else {
-		asprintf (&appres.file,
+		int ret;
+		ret = asprintf (&appres.file,
 			  "%s/.msgfile", getenv("HOME"));
-		if (appres.file == NULL)
+		if (ret == -1)
 		    errx (1, "cannot allocate memory for message");
 	    }
 	} else if(strcmp(argv[i], "--version") == 0) {
@@ -1029,7 +1026,7 @@ main (int argc, char **argv)
 #endif
 
     override = XtVaAppInitialize(&app, "XNlock", options, XtNumber(options),
-				 (Cardinal*)&argc, argv, NULL, 
+				 &argc, argv, NULL, 
 				 XtNoverrideRedirect, True, 
 				 NULL);
     
