@@ -1,13 +1,11 @@
 #!/bin/sh
 # check library exported symbols
-# $Id: check-symbols.sh,v 1.10.2.1 2005/06/15 01:25:48 lha Exp $
+# $Id: check-symbols.sh 19759 2007-01-08 11:46:08Z lha $
 
 LANG=C
 export LANG
 
-exit 0 #disable test on release branch
-
-esym=
+esym="__i686.get_pc_thunk fc_softc"
 symbols=
 
 # AIX has different default output format
@@ -38,7 +36,7 @@ while test $# != 0 ;do
 done
 
 for a in "$@" $esym; do
-    symbols="\$3 !~ /^_?${a}/ ${symbols:+&&} ${symbols}"
+    symbols="\$3 !~ /^[_\.]?${a}/ ${symbols:+&&} ${symbols}"
 done
 
 # F filename, N debugsymbols, W weak symbols, U undefined
