@@ -33,11 +33,14 @@
 
 #include "kdc_locl.h"
 
-RCSID("$Id: connect.c 19552 2006-12-28 21:09:00Z lha $");
+RCSID("$Id: connect.c 20958 2007-06-07 04:35:05Z lha $");
 
 /* Should we enable the HTTP hack? */
 int enable_http = -1;
 
+/* Log over requests to the KDC */
+const char *request_log;
+
 /* A string describing on what ports to listen */
 const char *port_str;
 
@@ -446,11 +449,15 @@ do_request(krb5_context context,
     krb5_data reply;
     int datagram_reply = (d->type == SOCK_DGRAM);
 
+    krb5_kdc_update_time(NULL);
+
     krb5_data_zero(&reply);
     ret = krb5_kdc_process_request(context, config, 
 				   buf, len, &reply, &prependlength,
 				   d->addr_string, d->sa,
 				   datagram_reply);
+    if(request_log)
+	krb5_kdc_save_request(context, request_log, buf, len, &reply, d->sa);
     if(reply.length){
 	send_reply(context, config, prependlength, d, &reply);
 	krb5_data_free(&reply);
