@@ -32,7 +32,7 @@
  */
 
 #include "kpasswd_locl.h"
-RCSID("$Id: kpasswdd.c 17571 2006-05-13 18:51:41Z lha $");
+RCSID("$Id: kpasswdd.c 21511 2007-07-12 12:43:46Z lha $");
 
 #include <kadm5/admin.h>
 #ifdef HAVE_SYS_UN_H
@@ -123,18 +123,20 @@ make_result (krb5_data *data,
 	     uint16_t result_code,
 	     const char *expl)
 {
+    char *str;
     krb5_data_zero (data);
 
-    data->length = asprintf ((char **)&data->data,
+    data->length = asprintf (&str,
 			     "%c%c%s",
 			     (result_code >> 8) & 0xFF,
 			     result_code & 0xFF,
 			     expl);
 
-    if (data->data == NULL) {
+    if (str == NULL) {
 	krb5_warnx (context, "Out of memory generating error reply");
 	return 1;
-   }
+    }
+    data->data = str;
     return 0;
 }
 
