@@ -36,6 +36,13 @@ hx509_ca_sign_self (
 	hx509_private_key /*signer*/,
 	hx509_cert */*certificate*/);
 
+int
+hx509_ca_tbs_add_crl_dp_uri (
+	hx509_context /*context*/,
+	hx509_ca_tbs /*tbs*/,
+	const char */*uri*/,
+	hx509_name /*issuername*/);
+
 int
 hx509_ca_tbs_add_eku (
 	hx509_context /*context*/,
@@ -54,6 +61,12 @@ hx509_ca_tbs_add_san_jid (
 	hx509_ca_tbs /*tbs*/,
 	const char */*jid*/);
 
+int
+hx509_ca_tbs_add_san_ms_upn (
+	hx509_context /*context*/,
+	hx509_ca_tbs /*tbs*/,
+	const char */*principal*/);
+
 int
 hx509_ca_tbs_add_san_otherName (
 	hx509_context /*context*/,
@@ -87,6 +100,11 @@ hx509_ca_tbs_set_ca (
 	hx509_ca_tbs /*tbs*/,
 	int /*pathLenConstraint*/);
 
+int
+hx509_ca_tbs_set_domaincontroller (
+	hx509_context /*context*/,
+	hx509_ca_tbs /*tbs*/);
+
 int
 hx509_ca_tbs_set_notAfter (
 	hx509_context /*context*/,
@@ -218,6 +236,13 @@ hx509_cert_init (
 	const Certificate */*c*/,
 	hx509_cert */*cert*/);
 
+int
+hx509_cert_init_data (
+	hx509_context /*context*/,
+	const void */*ptr*/,
+	size_t /*len*/,
+	hx509_cert */*cert*/);
+
 int
 hx509_cert_keyusage_print (
 	hx509_context /*context*/,
@@ -399,6 +424,41 @@ hx509_context_set_missing_revoke (
 	hx509_context /*context*/,
 	int /*flag*/);
 
+int
+hx509_crl_add_revoked_certs (
+	hx509_context /*context*/,
+	hx509_crl /*crl*/,
+	hx509_certs /*certs*/);
+
+int
+hx509_crl_alloc (
+	hx509_context /*context*/,
+	hx509_crl */*crl*/);
+
+void
+hx509_crl_free (
+	hx509_context /*context*/,
+	hx509_crl */*crl*/);
+
+int
+hx509_crl_lifetime (
+	hx509_context /*context*/,
+	hx509_crl /*crl*/,
+	int /*delta*/);
+
+int
+hx509_crl_sign (
+	hx509_context /*context*/,
+	hx509_cert /*signer*/,
+	hx509_crl /*crl*/,
+	heim_octet_string */*os*/);
+
+const AlgorithmIdentifier *
+hx509_crypto_aes128_cbc (void);
+
+const AlgorithmIdentifier *
+hx509_crypto_aes256_cbc (void);
+
 int
 hx509_crypto_available (
 	hx509_context /*context*/,
@@ -415,6 +475,9 @@ hx509_crypto_decrypt (
 	heim_octet_string */*ivec*/,
 	heim_octet_string */*clear*/);
 
+const AlgorithmIdentifier *
+hx509_crypto_des_rsdi_ede3_cbc (void);
+
 void
 hx509_crypto_destroy (hx509_crypto /*crypto*/);
 
@@ -423,7 +486,7 @@ hx509_crypto_encrypt (
 	hx509_crypto /*crypto*/,
 	const void */*data*/,
 	const size_t /*length*/,
-	heim_octet_string */*ivec*/,
+	const heim_octet_string */*ivec*/,
 	heim_octet_string **/*ciphertext*/);
 
 const heim_oid *
@@ -451,6 +514,11 @@ hx509_crypto_init (
 const char *
 hx509_crypto_provider (hx509_crypto /*crypto*/);
 
+int
+hx509_crypto_random_iv (
+	hx509_crypto /*crypto*/,
+	heim_octet_string */*ivec*/);
+
 int
 hx509_crypto_select (
 	const hx509_context /*context*/,
@@ -515,6 +583,11 @@ hx509_err (
 void
 hx509_free_octet_string_list (hx509_octet_string_list */*list*/);
 
+int
+hx509_general_name_unparse (
+	GeneralName */*name*/,
+	char **/*str*/);
+
 char *
 hx509_get_error_string (
 	hx509_context /*context*/,
@@ -664,7 +737,7 @@ hx509_peer_info_alloc (
 	hx509_context /*context*/,
 	hx509_peer_info */*peer*/);
 
-int
+void
 hx509_peer_info_free (hx509_peer_info /*peer*/);
 
 int
@@ -679,6 +752,36 @@ hx509_peer_info_set_cms_algs (
 	const AlgorithmIdentifier */*val*/,
 	size_t /*len*/);
 
+int
+hx509_pem_add_header (
+	hx509_pem_header **/*headers*/,
+	const char */*header*/,
+	const char */*value*/);
+
+const char *
+hx509_pem_find_header (
+	const hx509_pem_header */*h*/,
+	const char */*header*/);
+
+void
+hx509_pem_free_header (hx509_pem_header */*headers*/);
+
+int
+hx509_pem_read (
+	hx509_context /*context*/,
+	FILE */*f*/,
+	hx509_pem_read_func /*func*/,
+	void */*ctx*/);
+
+int
+hx509_pem_write (
+	hx509_context /*context*/,
+	const char */*type*/,
+	hx509_pem_header */*headers*/,
+	FILE */*f*/,
+	const void */*data*/,
+	size_t /*size*/);
+
 void
 hx509_print_func (
 	hx509_vprint_func /*func*/,
@@ -727,6 +830,17 @@ hx509_query_match_option (
 	hx509_query */*q*/,
 	hx509_query_option /*option*/);
 
+void
+hx509_query_statistic_file (
+	hx509_context /*context*/,
+	const char */*fn*/);
+
+void
+hx509_query_unparse_stats (
+	hx509_context /*context*/,
+	int /*printtype*/,
+	FILE */*out*/);
+
 int
 hx509_revoke_add_crl (
 	hx509_context /*context*/,
@@ -858,6 +972,11 @@ hx509_verify_attach_revoke (
 	hx509_verify_ctx /*ctx*/,
 	hx509_revoke_ctx /*revoke_ctx*/);
 
+void
+hx509_verify_ctx_f_allow_default_trustanchors (
+	hx509_verify_ctx /*ctx*/,
+	int /*boolean*/);
+
 void
 hx509_verify_destroy_ctx (hx509_verify_ctx /*ctx*/);
 
@@ -865,7 +984,8 @@ int
 hx509_verify_hostname (
 	hx509_context /*context*/,
 	const hx509_cert /*cert*/,
-	int /*require_match*/,
+	int /*flags*/,
+	hx509_hostname_type /*type*/,
 	const char */*hostname*/,
 	const struct sockaddr */*sa*/,
 	int /*sa_size*/);
@@ -882,6 +1002,11 @@ hx509_verify_path (
 	hx509_cert /*cert*/,
 	hx509_certs /*pool*/);
 
+void
+hx509_verify_set_max_depth (
+	hx509_verify_ctx /*ctx*/,
+	unsigned int /*max_depth*/);
+
 void
 hx509_verify_set_proxy_certificate (
 	hx509_verify_ctx /*ctx*/,
