@@ -32,7 +32,7 @@
  */
 
 #include "hx_locl.h"
-RCSID("$Id: hxtool.c 20438 2007-04-19 15:18:07Z lha $");
+RCSID("$Id: hxtool.c 21342 2007-06-26 14:21:34Z lha $");
 
 #include <hxtool-commands.h>
 #include <sl.h>
@@ -40,10 +40,12 @@ RCSID("$Id: hxtool.c 20438 2007-04-19 15:18:07Z lha $");
 
 static hx509_context context;
 
+static char *stat_file_string;
 static int version_flag;
 static int help_flag;
 
 struct getargs args[] = {
+    { "statistic-file", 0, arg_string, &stat_file_string },
     { "version", 0, arg_flag, &version_flag },
     { "help", 0, arg_flag, &help_flag }
 };
@@ -53,9 +55,14 @@ static void
 usage(int code)
 {
     arg_printusage(args, num_args, NULL, "command");
+    printf("Use \"%s help\" to get more help\n", getprogname());
     exit(code);
 }
 
+/*
+ *
+ */
+
 static void
 lock_strings(hx509_lock lock, getarg_strings *pass)
 {
@@ -68,6 +75,10 @@ lock_strings(hx509_lock lock, getarg_strings *pass)
     }
 }
 
+/*
+ *
+ */
+
 static void
 certs_strings(hx509_context context, const char *type, hx509_certs certs,
 	      hx509_lock lock, const getarg_strings *s)
@@ -82,6 +93,58 @@ certs_strings(hx509_context context, const char *type, hx509_certs certs,
     }
 }
 
+/*
+ *
+ */
+
+static void
+parse_oid(const char *str, const heim_oid *def, heim_oid *oid)
+{
+    int ret;
+    if (str)
+	ret = der_parse_heim_oid (str, " .", oid);
+    else
+	ret = der_copy_oid(def, oid);
+    if  (ret)
+	errx(1, "parse_oid failed for: %s", str ? str : "default oid");
+}
+
+/*
+ *
+ */
+
+static void
+peer_strings(hx509_context context,
+	     hx509_peer_info *peer, 
+	     const getarg_strings *s)
+{
+    AlgorithmIdentifier *val;
+    int ret, i;
+    
+    ret = hx509_peer_info_alloc(context, peer);
+    if (ret)
+	hx509_err(context, 1, ret, "hx509_peer_info_alloc");
+    
+    val = calloc(s->num_strings, sizeof(*val));
+    if (val == NULL)
+	err(1, "malloc");
+
+    for (i = 0; i < s->num_strings; i++)
+	parse_oid(s->strings[i], NULL, &val[i].algorithm);
+	    
+    ret = hx509_peer_info_set_cms_algs(context, *peer, val, s->num_strings);
+    if (ret)
+	hx509_err(context, 1, ret, "hx509_peer_info_set_cms_algs");
+
+    for (i = 0; i < s->num_strings; i++)
+	free_AlgorithmIdentifier(&val[i]);
+    free(val);
+}
+
+/*
+ *
+ */
+
 int
 cms_verify_sd(struct cms_verify_sd_options *opt, int argc, char **argv)
 {
@@ -182,7 +245,8 @@ cms_verify_sd(struct cms_verify_sd_options *opt, int argc, char **argv)
 int
 cms_create_sd(struct cms_create_sd_options *opt, int argc, char **argv)
 {
-    const heim_oid *contentType;
+    heim_oid contentType;
+    hx509_peer_info peer = NULL;
     heim_octet_string o;
     hx509_query *q;
     hx509_lock lock;
@@ -191,8 +255,9 @@ cms_create_sd(struct cms_create_sd_options *opt, int argc, char **argv)
     size_t sz;
     void *p;
     int ret, flags = 0;
+    char *signer_name = NULL;
 
-    contentType = oid_id_pkcs7_data();
+    memset(&contentType, 0, sizeof(contentType));
 
     if (argc < 2)
 	errx(1, "argc < 2");
@@ -215,6 +280,8 @@ cms_create_sd(struct cms_create_sd_options *opt, int argc, char **argv)
 
     if (opt->detached_signature_flag)
 	flags |= HX509_CMS_SIGATURE_DETACHED;
+    if (opt->id_by_name_flag)
+	flags |= HX509_CMS_SIGATURE_ID_NAME;
 
     ret = hx509_query_alloc(context, &q);
     if (ret)
@@ -229,32 +296,53 @@ cms_create_sd(struct cms_create_sd_options *opt, int argc, char **argv)
     ret = hx509_certs_find(context, store, q, &cert);
     hx509_query_free(context, q);
     if (ret)
-	errx(1, "hx509_certs_find: %d", ret);
+	hx509_err(context, 1, ret, "hx509_certs_find");
 
     ret = _hx509_map_file(argv[0], &p, &sz, NULL);
     if (ret)
 	err(1, "map_file: %s: %d", argv[0], ret);
 
+    if (opt->peer_alg_strings.num_strings)
+	peer_strings(context, &peer, &opt->peer_alg_strings);
+
+    parse_oid(opt->content_type_string, oid_id_pkcs7_data(), &contentType);
+
     ret = hx509_cms_create_signed_1(context,
 				    flags,
-				    contentType,
+				    &contentType,
 				    p,
 				    sz, 
 				    NULL,
 				    cert,
-				    NULL,
+				    peer,
 				    anchors,
 				    pool,
 				    &o);
     if (ret)
 	errx(1, "hx509_cms_create_signed: %d", ret);
 
+    {
+	hx509_name name;
+	
+	ret = hx509_cert_get_subject(cert, &name);
+	if (ret)
+	    errx(1, "hx509_cert_get_subject");
+	
+	ret = hx509_name_to_string(name, &signer_name);
+	hx509_name_free(&name);
+	if (ret)
+	    errx(1, "hx509_name_to_string");
+    }
+
+
     hx509_certs_free(&anchors);
     hx509_certs_free(&pool);
     hx509_cert_free(cert);
     hx509_certs_free(&store);
     _hx509_unmap_file(p, sz);
     hx509_lock_free(lock);
+    hx509_peer_info_free(peer);
+    der_free_oid(&contentType);
 
     if (opt->content_info_flag) {
 	heim_octet_string wo;
@@ -267,10 +355,32 @@ cms_create_sd(struct cms_create_sd_options *opt, int argc, char **argv)
 	o = wo;
     }
 
-    ret = _hx509_write_file(argv[1], o.data, o.length);
-    if (ret)
-	errx(1, "hx509_write_file: %d", ret);
+    if (opt->pem_flag) {
+	hx509_pem_header *header = NULL;
+	FILE *f;
 
+	hx509_pem_add_header(&header, "Content-disposition", 
+			     opt->detached_signature_flag ? "detached" : "inline");
+	hx509_pem_add_header(&header, "Signer", signer_name);
+
+	f = fopen(argv[1], "w");
+	if (f == NULL)
+	    err(1, "open %s", argv[1]);
+	
+	ret = hx509_pem_write(context, "CMS SIGNEDDATA", header, f, 
+			      o.data, o.length);
+	fclose(f);
+	hx509_pem_free_header(header);
+	if (ret)
+	    errx(1, "hx509_pem_write: %d", ret);
+
+    } else {
+	ret = _hx509_write_file(argv[1], o.data, o.length);
+	if (ret)
+	    errx(1, "hx509_write_file: %d", ret);
+    }
+
+    free(signer_name);
     free(o.data);
 
     return 0;
@@ -342,6 +452,7 @@ cms_unenvelope(struct cms_unenvelope_options *opt, int argc, char **argv)
 int
 cms_create_enveloped(struct cms_envelope_options *opt, int argc, char **argv)
 {
+    heim_oid contentType;
     heim_octet_string o;
     const heim_oid *enctype = NULL;
     hx509_query *q;
@@ -352,6 +463,8 @@ cms_create_enveloped(struct cms_envelope_options *opt, int argc, char **argv)
     void *p;
     hx509_lock lock;
 
+    memset(&contentType, 0, sizeof(contentType));
+
     hx509_lock_init(context, &lock);
     lock_strings(lock, &opt->pass_strings);
 
@@ -381,14 +494,17 @@ cms_create_enveloped(struct cms_envelope_options *opt, int argc, char **argv)
     if (ret)
 	errx(1, "hx509_certs_find: %d", ret);
 
+    parse_oid(opt->content_type_string, oid_id_pkcs7_data(), &contentType);
+
     ret = hx509_cms_envelope_1(context, 0, cert, p, sz, enctype, 
-			       oid_id_pkcs7_data(), &o);
+			       &contentType, &o);
     if (ret)
 	errx(1, "hx509_cms_envelope_1: %d", ret);
 
     hx509_cert_free(cert);
     hx509_certs_free(&certs);
     _hx509_unmap_file(p, sz);
+    der_free_oid(&contentType);
 
     if (opt->content_info_flag) {
 	heim_octet_string wo;
@@ -465,6 +581,8 @@ print_certificate(hx509_context hxcontext, hx509_cert cert, int verbose)
 	hx509_validate_ctx_add_flags(vctx, HX509_VALIDATE_F_VERBOSE);
 	
 	hx509_validate_cert(hxcontext, vctx, cert);
+
+	hx509_validate_ctx_free(vctx);
     }
 }
 
@@ -581,6 +699,7 @@ certificate_copy(struct certificate_copy_options *opt, int argc, char **argv)
 	    hx509_err(context, 1, ret, "hx509_certs_store");
 
     hx509_certs_free(&certs);
+    hx509_lock_free(lock);
 
     return 0;
 }
@@ -588,6 +707,8 @@ certificate_copy(struct certificate_copy_options *opt, int argc, char **argv)
 struct verify {
     hx509_verify_ctx ctx;
     hx509_certs chain;
+    const char *hostname;
+    int errors;
 };
 
 static int
@@ -601,10 +722,20 @@ verify_f(hx509_context hxcontext, void *ctx, hx509_cert c)
 	char *s = hx509_get_error_string(hxcontext, ret);
 	printf("verify_path: %s: %d\n", s, ret);
 	free(s);
+	v->errors++;
     } else
 	printf("path ok\n");
 
-    return ret;
+    if (v->hostname) {
+	ret = hx509_verify_hostname(hxcontext, c, 0, HX509_HN_HOSTNAME,
+				    v->hostname, NULL, 0);
+	if (ret) {
+	    printf("verify_hostname: %d\n", ret);
+	    v->errors++;
+	}
+    }
+
+    return 0;
 }
 
 int
@@ -616,6 +747,8 @@ pcert_verify(struct verify_options *opt, int argc, char **argv)
     struct verify v;
     int ret;
 
+    memset(&v, 0, sizeof(v));
+
     if (opt->missing_revoke_flag)
 	hx509_context_set_missing_revoke(context, 1);
 
@@ -644,6 +777,11 @@ pcert_verify(struct verify_options *opt, int argc, char **argv)
 	hx509_verify_set_time(ctx, t);
     }
 
+    if (opt->hostname_string)
+	v.hostname = opt->hostname_string;
+    if (opt->max_depth_integer)
+	hx509_verify_set_max_depth(ctx, opt->max_depth_integer);
+
     ret = hx509_revoke_init(context, &revoke_ctx);
     if (ret)
 	errx(1, "hx509_revoke_init: %d", ret);
@@ -656,14 +794,14 @@ pcert_verify(struct verify_options *opt, int argc, char **argv)
 
 	    ret = hx509_certs_append(context, chain, NULL, s);
 	    if (ret)
-		errx(1, "hx509_certs_append: chain: %s: %d", s, ret);
+		hx509_err(context, 1, ret, "hx509_certs_append: chain: %s: %d", s, ret);
 
 	} else if (strncmp(s, "anchor:", 7) == 0) {
 	    s += 7;
 
 	    ret = hx509_certs_append(context, anchors, NULL, s);
 	    if (ret)
-		errx(1, "hx509_certs_append: anchor: %s: %d", s, ret);
+		hx509_err(context, 1, ret, "hx509_certs_append: anchor: %s: %d", s, ret);
 
 	} else if (strncmp(s, "cert:", 5) == 0) {
 	    s += 5;
@@ -698,7 +836,7 @@ pcert_verify(struct verify_options *opt, int argc, char **argv)
     v.ctx = ctx;
     v.chain = chain;
 
-    ret = hx509_certs_iter(context, certs, verify_f, &v);
+    hx509_certs_iter(context, certs, verify_f, &v);
 
     hx509_verify_destroy_ctx(ctx);
 
@@ -708,7 +846,12 @@ pcert_verify(struct verify_options *opt, int argc, char **argv)
 
     hx509_revoke_free(&revoke_ctx);
 
-    return ret;
+    if (v.errors) {
+	printf("failed verifing %d checks\n", v.errors);
+	return 1;
+    }
+
+    return 0;
 }
 
 int
@@ -843,6 +986,64 @@ ocsp_print(struct ocsp_print_options *opt, int argc, char **argv)
     return 0;
 }
 
+/*
+ *
+ */
+
+static int
+verify_o(hx509_context hxcontext, void *ctx, hx509_cert c)
+{
+    heim_octet_string *os = ctx;
+    time_t expiration;
+    int ret;
+
+    ret = hx509_ocsp_verify(context, 0, c, 0, 
+			    os->data, os->length, &expiration);
+    if (ret) {
+	char *s = hx509_get_error_string(hxcontext, ret);
+	printf("ocsp_verify: %s: %d\n", s, ret);
+	free(s);
+    } else
+	printf("expire: %d\n", (int)expiration);
+
+    return ret;
+}
+
+
+int
+ocsp_verify(struct ocsp_verify_options *opt, int argc, char **argv)
+{
+    hx509_lock lock;
+    hx509_certs certs;
+    int ret, i;
+    heim_octet_string os;
+    
+    hx509_lock_init(context, &lock);
+
+    if (opt->ocsp_file_string == NULL)
+	errx(1, "no ocsp file given");
+
+    ret = _hx509_map_file(opt->ocsp_file_string, &os.data, &os.length, NULL);
+    if (ret)
+	err(1, "map_file: %s: %d", argv[0], ret);
+    
+    ret = hx509_certs_init(context, "MEMORY:test-certs", 0, NULL, &certs);
+
+    for (i = 0; i < argc; i++) {
+	ret = hx509_certs_append(context, certs, lock, argv[i]);
+	if (ret)
+	    hx509_err(context, 1, ret, "hx509_certs_append: %s", argv[i]);
+    }
+
+    ret = hx509_certs_iter(context, certs, verify_o, &os);
+
+    hx509_certs_free(&certs);
+    _hx509_unmap_file(os.data, os.length);
+    hx509_lock_free(lock);
+
+    return ret;
+}
+
 static int
 read_private_key(const char *fn, hx509_private_key *key)
 {
@@ -1001,60 +1202,23 @@ request_create(struct request_create_options *opt, int argc, char **argv)
 }
 
 int
-pkcs10_print(struct pkcs10_print_options *opt, int argc, char **argv)
+request_print(struct request_print_options *opt, int argc, char **argv)
 {
-    size_t size, length;
     int ret, i;
-    void *p;
 
-    printf("pkcs10 print\n");
+    printf("request print\n");
 
     for (i = 0; i < argc; i++) {
-	CertificationRequest req;
-	CertificationRequestInfo *rinfo;
+	hx509_request req;
 
-	ret = _hx509_map_file(argv[i], &p, &length, NULL);
+	ret = _hx509_request_parse(context, argv[i], &req);
 	if (ret)
-	    err(1, "map_file: %s: %d", argv[i], ret);
+	    hx509_err(context, 1, ret, "parse_request: %s", argv[i]);
 
-	ret = decode_CertificationRequest(p, length, &req, &size);
-	_hx509_unmap_file(p, length);
+	ret = _hx509_request_print(context, req, stdout);
+	_hx509_request_free(&req);
 	if (ret)
-	    errx(1, "failed to parse file %s: %d", argv[i], ret);
-
-	rinfo = &req.certificationRequestInfo;
-
-	{
-	    char *subject;
-	    hx509_name n;
-
-	    ret = _hx509_name_from_Name(&rinfo->subject, &n);
-	    if (ret)
-		abort();
-	    
-	    ret = hx509_name_to_string(n, &subject);
-	    hx509_name_free(&n);
-	    if (ret)
-		abort();
-	    
-	    printf("name: %s\n", subject);
-	    free(subject);
-	}
-
-	if (rinfo->attributes && rinfo->attributes->len) {
-	    int j;
-
-	    printf("Attributes:\n");
-
-	    for (j = 0; j < rinfo->attributes->len; j++) {
-		char *str;
-		hx509_oid_sprint(&rinfo->attributes->val[j].type, &str);
-		printf("\toid: %s\n", str);
-		free(str);
-	    }
-	}
-
-	free_CertificationRequest(&req);
+	    hx509_err(context, 1, ret, "Failed to print file %s", argv[i]);
     }
 
     return 0;
@@ -1131,6 +1295,8 @@ crypto_available(struct crypto_available_options *opt, int argc, char **argv)
 	    type = HX509_SELECT_DIGEST;
 	else if (strcmp(opt->type_string, "public-sig") == 0)
 	    type = HX509_SELECT_PUBLIC_SIG;
+	else if (strcmp(opt->type_string, "secret") == 0)
+	    type = HX509_SELECT_SECRET_ENC;
 	else
 	    errx(1, "unknown type: %s", opt->type_string);
     } else
@@ -1165,37 +1331,15 @@ crypto_select(struct crypto_select_options *opt, int argc, char **argv)
 	    type = HX509_SELECT_DIGEST;
 	else if (strcmp(opt->type_string, "public-sig") == 0)
 	    type = HX509_SELECT_PUBLIC_SIG;
+	else if (strcmp(opt->type_string, "secret") == 0)
+	    type = HX509_SELECT_SECRET_ENC;
 	else
 	    errx(1, "unknown type: %s", opt->type_string);
     } else
 	type = HX509_SELECT_DIGEST;
 
-    if (opt->peer_cmstype_strings.num_strings) {
-	AlgorithmIdentifier *val;
-	size_t i;
-
-	ret = hx509_peer_info_alloc(context, &peer);
-	if (ret)
-	    errx(1, "hx509_peer_info_alloc");
-
-	val = calloc(opt->peer_cmstype_strings.num_strings, sizeof(*val));
-	if (val == NULL)
-	    err(1, "malloc");
-
-	for (i = 0; i < opt->peer_cmstype_strings.num_strings; i++) {
-	    ret = der_parse_heim_oid (opt->peer_cmstype_strings.strings[i],
-				      " .", &val[i].algorithm);
-	    if  (ret)
-		errx(1, "der_parse_heim_oid failed on: %s", 
-		     opt->peer_cmstype_strings.strings[i]);
-	}
-	    
-	ret = hx509_peer_info_set_cms_algs(context, peer, val, 
-					   opt->peer_cmstype_strings.num_strings);
-	if (ret)
-	    errx(1, "hx509_peer_info_set_cms_algs");
-
-    }
+    if (opt->peer_cmstype_strings.num_strings)
+	peer_strings(context, &peer, &opt->peer_cmstype_strings);
 
     ret = hx509_crypto_select(context, type, NULL, peer, &selected);
     if (ret)
@@ -1206,8 +1350,7 @@ crypto_select(struct crypto_select_options *opt, int argc, char **argv)
     free(s);
     free_AlgorithmIdentifier(&selected);
 
-    if (peer)
-	hx509_peer_info_free(peer);
+    hx509_peer_info_free(peer);
 
     return 0;
 }
@@ -1265,6 +1408,11 @@ eval_types(hx509_context context,
 				       oid_id_pkix_kp_clientAuth());
 	    if (ret)
 		hx509_err(context, 1, ret, "hx509_ca_tbs_add_eku");
+	} else if (strcmp(type, "peap-server") == 0) {
+	    ret = hx509_ca_tbs_add_eku(context, tbs, 
+				       oid_id_pkix_kp_serverAuth());
+	    if (ret)
+		hx509_err(context, 1, ret, "hx509_ca_tbs_add_eku");
 	} else if (strcmp(type, "pkinit-kdc") == 0) {
 	    pkinit++;
 	    ret = hx509_ca_tbs_add_eku(context, tbs, 
@@ -1277,6 +1425,17 @@ eval_types(hx509_context context,
 				       oid_id_pkekuoid());
 	    if (ret)
 		hx509_err(context, 1, ret, "hx509_ca_tbs_add_eku");
+
+	    ret = hx509_ca_tbs_add_eku(context, tbs, 
+				       oid_id_ms_client_authentication());
+	    if (ret)
+		hx509_err(context, 1, ret, "hx509_ca_tbs_add_eku");
+
+	    ret = hx509_ca_tbs_add_eku(context, tbs, 
+				       oid_id_pkinit_ms_eku());
+	    if (ret)
+		hx509_err(context, 1, ret, "hx509_ca_tbs_add_eku");
+
 	} else if (strcmp(type, "email") == 0) {
 	    ret = hx509_ca_tbs_add_eku(context, tbs, 
 				       oid_id_pkix_kp_emailProtection());
@@ -1298,6 +1457,16 @@ eval_types(hx509_context context,
 	if (ret)
 	    hx509_err(context, 1, ret, "hx509_ca_tbs_add_san_pkinit");
     }
+
+    if (opt->ms_upn_string) {
+	if (!pkinit)
+	    errx(1, "MS up given but no pk-init oid");
+
+	ret = hx509_ca_tbs_add_san_ms_upn(context, tbs, opt->ms_upn_string);
+	if (ret)
+	    hx509_err(context, 1, ret, "hx509_ca_tbs_add_san_ms_upn");
+    }
+
     
     for (i = 0; i < opt->hostname_strings.num_strings; i++) {
 	const char *hostname = opt->hostname_strings.strings[i];
@@ -1406,31 +1575,18 @@ hxtool_ca(struct certificate_sign_options *opt, int argc, char **argv)
     }
 
     if (opt->req_string) {
-	CertificationRequest req;
-	CertificationRequestInfo *rinfo;
-	void *p;
-	size_t len, size;
+	hx509_request req;
 
-	ret = _hx509_map_file(opt->req_string, &p, &len, NULL);
+	ret = _hx509_request_parse(context, opt->req_string, &req);
 	if (ret)
-	    err(1, "map_file: %s: %d", opt->req_string, ret);
-
-	ret = decode_CertificationRequest(p, len, &req, &size);
-	_hx509_unmap_file(p, len);
+	    hx509_err(context, 1, ret, "parse_request: %s", opt->req_string);
+	ret = _hx509_request_get_name(context, req, &subject);
 	if (ret)
-	    errx(1, "failed to parse req file %s: %d", opt->req_string, ret);
-
-	rinfo = &req.certificationRequestInfo;
-
-	ret = _hx509_name_from_Name(&rinfo->subject, &subject);
+	    hx509_err(context, 1, ret, "get name");
+	ret = _hx509_request_get_SubjectPublicKeyInfo(context, req, &spki);
 	if (ret)
-	    errx(1, "_hx509_name_from_Name %d", ret);
-
-	ret = copy_SubjectPublicKeyInfo(&rinfo->subjectPKInfo, &spki);
-	if (ret)
-	    errx(1, "copy_SubjectPublicKeyInfo: %d", ret);
-
-	free_CertificationRequest(&req);
+	    hx509_err(context, 1, ret, "get spki");
+	_hx509_request_free(&req);
     }
 
     if (opt->generate_key_string) {
@@ -1535,6 +1691,13 @@ hxtool_ca(struct certificate_sign_options *opt, int argc, char **argv)
 	    hx509_err(context, 1, ret, "hx509_ca_tbs_set_subject");
     }
 
+    if (opt->crl_uri_string) {
+	ret = hx509_ca_tbs_add_crl_dp_uri(context, tbs, 
+					  opt->crl_uri_string, NULL);
+	if (ret)
+	    hx509_err(context, 1, ret, "hx509_ca_tbs_add_crl_dp_uri");
+    }
+
     eval_types(context, tbs, opt);
 
     if (opt->issue_ca_flag) {
@@ -1547,6 +1710,11 @@ hxtool_ca(struct certificate_sign_options *opt, int argc, char **argv)
 	if (ret)
 	    hx509_err(context, 1, ret, "hx509_ca_tbs_set_proxy");
     }
+    if (opt->domain_controller_flag) {
+	hx509_ca_tbs_set_domaincontroller(context, tbs);
+	if (ret)
+	    hx509_err(context, 1, ret, "hx509_ca_tbs_set_domaincontroller");
+    }
 
     if (delta) {
 	ret = hx509_ca_tbs_set_notAfter_lifetime(context, tbs, delta);
@@ -1605,6 +1773,173 @@ hxtool_ca(struct certificate_sign_options *opt, int argc, char **argv)
     return 0;
 }
 
+static int
+test_one_cert(hx509_context hxcontext, void *ctx, hx509_cert cert)
+{
+    heim_octet_string sd, c;
+    hx509_verify_ctx vctx = ctx;
+    hx509_certs signer = NULL;
+    heim_oid type;
+    int ret;
+
+    if (_hx509_cert_private_key(cert) == NULL)
+	return 0;
+
+    ret = hx509_cms_create_signed_1(context, 0, NULL, NULL, 0,
+				    NULL, cert, NULL, NULL, NULL, &sd);
+    if (ret)
+	errx(1, "hx509_cms_create_signed_1");
+
+    ret = hx509_cms_verify_signed(context, vctx, sd.data, sd.length,
+				  NULL, NULL, &type, &c, &signer);
+    free(sd.data);
+    if (ret)
+	hx509_err(context, 1, ret, "hx509_cms_verify_signed");
+
+    printf("create-signature verify-sigature done\n");
+
+    free(c.data);
+
+    return 0;
+}
+
+int
+test_crypto(struct test_crypto_options *opt, int argc, char ** argv)
+{
+    hx509_verify_ctx vctx;
+    hx509_certs certs;
+    hx509_lock lock;
+    int i, ret;
+
+    hx509_lock_init(context, &lock);
+    lock_strings(lock, &opt->pass_strings);
+
+    ret = hx509_certs_init(context, "MEMORY:test-crypto", 0, NULL, &certs);
+
+    for (i = 0; i < argc; i++) {
+	ret = hx509_certs_append(context, certs, lock, argv[i]);
+	if (ret)
+	    hx509_err(context, 1, ret, "hx509_certs_append");
+    }
+
+    ret = hx509_verify_init_ctx(context, &vctx);
+    if (ret)
+	hx509_err(context, 1, ret, "hx509_verify_init_ctx");
+
+    hx509_verify_attach_anchors(vctx, certs);
+
+    ret = hx509_certs_iter(context, certs, test_one_cert, vctx);
+
+    hx509_certs_free(&certs);
+
+    return 0;
+}
+
+int
+statistic_print(struct statistic_print_options*opt, int argc, char **argv)
+{
+    int type = 0;
+
+    if (stat_file_string == NULL)
+	errx(1, "no stat file");
+
+    if (opt->type_integer)
+	type = opt->type_integer;
+
+    hx509_query_unparse_stats(context, type, stdout);
+    return 0;
+}
+
+/*
+ *
+ */
+
+int
+crl_sign(struct crl_sign_options *opt, int argc, char **argv)
+{
+    hx509_crl crl;
+    heim_octet_string os;
+    hx509_cert signer = NULL;
+    hx509_lock lock;
+    int ret;
+
+    hx509_lock_init(context, &lock);
+    lock_strings(lock, &opt->pass_strings);
+
+    ret = hx509_crl_alloc(context, &crl);
+    if (ret)
+	errx(1, "crl alloc");
+
+    if (opt->signer_string == NULL)
+	errx(1, "signer missing");
+
+    {
+	hx509_certs certs = NULL;
+	hx509_query *q;
+
+	ret = hx509_certs_init(context, opt->signer_string, 0,
+			       NULL, &certs);
+	if (ret)
+	    hx509_err(context, 1, ret, 
+		      "hx509_certs_init: %s", opt->signer_string);
+
+	ret = hx509_query_alloc(context, &q);
+	if (ret)
+	    hx509_err(context, 1, ret, "hx509_query_alloc: %d", ret);
+
+	hx509_query_match_option(q, HX509_QUERY_OPTION_PRIVATE_KEY);
+
+	ret = hx509_certs_find(context, certs, q, &signer);
+	hx509_query_free(context, q);
+	hx509_certs_free(&certs);
+	if (ret)
+	    hx509_err(context, 1, ret, "no signer certificate found");
+    }
+
+    if (opt->lifetime_string) {
+	int delta;
+
+	delta = parse_time(opt->lifetime_string, "day");
+	if (delta < 0)
+	    errx(1, "Invalid lifetime: %s", opt->lifetime_string);
+
+	hx509_crl_lifetime(context, crl, delta);
+    }
+
+    {
+	hx509_certs revoked = NULL;
+	int i;
+
+	ret = hx509_certs_init(context, "MEMORY:revoked-certs", 0,
+			       NULL, &revoked);
+
+	for (i = 0; i < argc; i++) {
+	    ret = hx509_certs_append(context, revoked, lock, argv[i]);
+	    if (ret)
+		hx509_err(context, 1, ret, "hx509_certs_append: %s", argv[i]);
+	}
+
+	hx509_crl_add_revoked_certs(context, crl, revoked);
+	hx509_certs_free(&revoked);
+    }
+
+    hx509_crl_sign(context, signer, crl, &os);
+
+    if (opt->crl_file_string)
+	rk_dumpdata(opt->crl_file_string, os.data, os.length);
+
+    free(os.data);
+
+    hx509_crl_free(context, &crl);
+    hx509_cert_free(signer);
+    hx509_lock_free(lock);
+
+    return 0;
+}
+
+/*
+ *
+ */
 
 int
 help(void *opt, int argc, char **argv)
@@ -1638,6 +1973,9 @@ main(int argc, char **argv)
     if (ret)
 	errx(1, "hx509_context_init failed with %d", ret);
 
+    if (stat_file_string)
+	hx509_query_statistic_file(context, stat_file_string);
+
     ret = sl_command(commands, argc, argv);
     if(ret == -1)
 	warnx ("unrecognized command: %s", argv[0]);
