@@ -31,7 +31,7 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: iprop.h 13131 2003-11-23 06:24:16Z lha $ */
+/* $Id: iprop.h 21767 2007-08-01 03:59:44Z lha $ */
 
 #ifndef __IPROP_H__
 #define __IPROP_H__
@@ -68,4 +68,7 @@ enum iprop_cmd { I_HAVE = 1,
 		 I_AM_HERE = 7
 };
 
+extern sig_atomic_t exit_flag;
+void setup_signal(void);
+
 #endif /* __IPROP_H__ */
