@@ -34,29 +34,34 @@
 #include "iprop.h"
 #include <rtbl.h>
 
-RCSID("$Id: ipropd_master.c 19233 2006-12-06 08:04:05Z lha $");
+RCSID("$Id: ipropd_master.c 21826 2007-08-06 02:21:48Z lha $");
 
 static krb5_log_facility *log_facility;
 
-const char *slave_stats_file = KADM5_SLAVE_STATS;
+const char *slave_stats_file;
 const char *slave_time_missing = "2 min";
 const char *slave_time_gone = "5 min";
 
 static int time_before_missing;
 static int time_before_gone;
 
+const char *master_hostname;
+
 static int
 make_signal_socket (krb5_context context)
 {
     struct sockaddr_un addr;
+    const char *fn;
     int fd;
 
+    fn = kadm5_log_signal_socket(context);
+
     fd = socket (AF_UNIX, SOCK_DGRAM, 0);
     if (fd < 0)
 	krb5_err (context, 1, errno, "socket AF_UNIX");
     memset (&addr, 0, sizeof(addr));
     addr.sun_family = AF_UNIX;
-    strlcpy (addr.sun_path, KADM5_LOG_SIGNAL, sizeof(addr.sun_path));
+    strlcpy (addr.sun_path, fn, sizeof(addr.sun_path));
     unlink (addr.sun_path);
     if (bind (fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
 	krb5_err (context, 1, errno, "bind %s", addr.sun_path);
@@ -119,11 +124,20 @@ typedef struct slave slave;
 static int
 check_acl (krb5_context context, const char *name)
 {
+    const char *fn;
     FILE *fp;
     char buf[256];
     int ret = 1;
 
-    fp = fopen (KADM5_SLAVE_ACL, "r");
+
+    fn = krb5_config_get_string_default(context,
+					NULL,
+					KADM5_SLAVE_ACL,
+					"kdc",
+					"iprop-acl",
+					NULL);
+
+    fp = fopen (fn, "r");
     if (fp == NULL)
 	return 1;
     while (fgets(buf, sizeof(buf), fp) != NULL) {
@@ -161,8 +175,10 @@ slave_gone_p (slave *s)
 }
 
 static void
-slave_dead(slave *s)
+slave_dead(krb5_context context, slave *s)
 {
+    krb5_warnx(context, "slave %s dead", s->name);
+
     if (s->fd >= 0) {
 	close (s->fd);
 	s->fd = -1;
@@ -215,7 +231,11 @@ add_slave (krb5_context context, krb5_keytab keytab, slave **root, int fd)
 	krb5_warn (context, errno, "accept");
 	goto error;
     }
-    gethostname(hostname, sizeof(hostname));
+    if (master_hostname)
+	strlcpy(hostname, master_hostname, sizeof(hostname));
+    else
+	gethostname(hostname, sizeof(hostname));
+
     ret = krb5_sname_to_principal (context, hostname, IPROP_NAME,
 				   KRB5_NT_SRV_HST, &server);
     if (ret) {
@@ -337,14 +357,14 @@ send_complete (krb5_context context, slave *s,
 
     if (ret) {
 	krb5_warn (context, ret, "krb5_write_priv_message");
-	slave_dead(s);
+	slave_dead(context, s);
 	return ret;
     }
 
     ret = hdb_foreach (context, db, 0, prop_one, s);
     if (ret) {
 	krb5_warn (context, ret, "hdb_foreach");
-	slave_dead(s);
+	slave_dead(context, s);
 	return ret;
     }
 
@@ -364,7 +384,7 @@ send_complete (krb5_context context, slave *s,
 
     ret = krb5_write_priv_message(context, s->ac, &s->fd, &data);
     if (ret) {
-	slave_dead(s);
+	slave_dead(context, s);
 	krb5_warn (context, ret, "krb5_write_priv_message");
 	return ret;
     }
@@ -393,7 +413,7 @@ send_are_you_there (krb5_context context, slave *s)
     sp = krb5_storage_from_mem (buf, 4);
     if (sp == NULL) {
 	krb5_warnx (context, "are_you_there: krb5_data_alloc");
-	slave_dead(s);
+	slave_dead(context, s);
 	return 1;
     }
     krb5_store_int32 (sp, ARE_YOU_THERE);
@@ -403,7 +423,7 @@ send_are_you_there (krb5_context context, slave *s)
 
     if (ret) {
 	krb5_warn (context, ret, "are_you_there: krb5_write_priv_message");
-	slave_dead(s);
+	slave_dead(context, s);
 	return 1;
     }
 
@@ -423,12 +443,22 @@ send_diffs (krb5_context context, slave *s, int log_fd,
     krb5_data data;
     int ret = 0;
 
-    if (s->version == current_version)
+    if (s->version == current_version) {
+	krb5_warnx(context, "slave %s in sync already at version %ld",
+		   s->name, (long)s->version);
 	return 0;
+    }
 
     if (s->flags & SLAVE_F_DEAD)
 	return 0;
 
+    /* if slave is a fresh client, starting over */
+    if (s->version == 0) {
+	krb5_warnx(context, "sending complete log to fresh slave %s",
+		   s->name);
+	return send_complete (context, s, database, current_version);
+    }
+
     sp = kadm5_log_goto_end (log_fd);
     right = krb5_storage_seek(sp, 0, SEEK_CUR);
     for (;;) {
@@ -441,13 +471,24 @@ send_diffs (krb5_context context, slave *s, int log_fd,
 	    return 0;
 	if (ver == s->version + 1)
 	    break;
-	if (left == 0)
+	if (left == 0) {
+	    krb5_warnx(context,
+		       "slave %s (version %lu) out of sync with master "
+		       "(first version in log %lu), sending complete database",
+		       s->name, (unsigned long)s->version, (unsigned long)ver);
 	    return send_complete (context, s, database, current_version);
+	}
     }
+
+    krb5_warnx(context,
+	       "syncing slave %s from version %lu to version %lu",
+	       s->name, (unsigned long)s->version,
+	       (unsigned long)current_version);
+
     ret = krb5_data_alloc (&data, right - left + 4);
     if (ret) {
 	krb5_warn (context, ret, "send_diffs: krb5_data_alloc");
-	slave_dead(s);
+	slave_dead(context, s);
 	return 1;
     }
     krb5_storage_read (sp, (char *)data.data + 4, data.length - 4);
@@ -456,7 +497,7 @@ send_diffs (krb5_context context, slave *s, int log_fd,
     sp = krb5_storage_from_data (&data);
     if (sp == NULL) {
 	krb5_warnx (context, "send_diffs: krb5_storage_from_data");
-	slave_dead(s);
+	slave_dead(context, s);
 	return 1;
     }
     krb5_store_int32 (sp, FOR_YOU);
@@ -467,11 +508,13 @@ send_diffs (krb5_context context, slave *s, int log_fd,
 
     if (ret) {
 	krb5_warn (context, ret, "send_diffs: krb5_write_priv_message");
-	slave_dead(s);
+	slave_dead(context, s);
 	return 1;
     }
     slave_seen(s);
 
+    s->version = current_version;
+
     return 0;
 }
 
@@ -508,8 +551,21 @@ process_msg (krb5_context context, slave *s, int log_fd,
 	    krb5_warnx (context, "process_msg: client send too I_HAVE data");
 	    break;
 	}
-	s->version = tmp;
-	ret = send_diffs (context, s, log_fd, database, current_version);
+	/* new started slave that have old log */
+	if (s->version == 0 && tmp != 0) {
+	    if (s->version < tmp) {
+		krb5_warnx (context, "Slave %s have later version the master "
+			    "OUT OF SYNC", s->name);
+	    } else {
+		s->version = tmp;
+	    }
+	}
+	if (tmp < s->version) {
+	    krb5_warnx (context, "Slave claims to not have "
+			"version we already sent to it");
+	} else {
+	    ret = send_diffs (context, s, log_fd, database, current_version);
+	}
 	break;
     case I_AM_HERE :
 	break;
@@ -533,6 +589,40 @@ process_msg (krb5_context context, slave *s, int log_fd,
 #define SLAVE_STATUS	"Status"
 #define SLAVE_SEEN	"Last Seen"
 
+static FILE *
+open_stats(krb5_context context)
+{
+    const char *fn;
+
+    if (slave_stats_file)
+	fn = slave_stats_file;
+    else
+	fn = krb5_config_get_string_default(context,
+					    NULL,
+					    KADM5_SLAVE_STATS,
+					    "kdc",
+					    "iprop-stats",
+					    NULL);
+
+    return fopen(fn, "w");
+}
+
+static void
+write_master_down(krb5_context context)
+{
+    char str[100];
+    time_t t = time(NULL);
+    FILE *fp;
+
+    fp = open_stats(context);
+    if (fp == NULL)
+	return;
+    krb5_format_time(context, t, str, sizeof(str), TRUE); 
+    fprintf(fp, "master down at %s\n", str);
+
+    fclose(fp);
+}
+
 static void
 write_stats(krb5_context context, slave *slaves, uint32_t current_version)
 {
@@ -541,7 +631,7 @@ write_stats(krb5_context context, slave *slaves, uint32_t current_version)
     time_t t = time(NULL);
     FILE *fp;
 
-    fp = fopen(slave_stats_file, "w");
+    fp = open_stats(context);
     if (fp == NULL)
 	return;
 
@@ -624,6 +714,8 @@ static struct getargs args[] = {
       "port ipropd will listen to", "port"},
     { "detach", 0, arg_flag, &detach_from_console, 
       "detach from console" },
+    { "hostname", 0, arg_string, &master_hostname, 
+      "hostname of master (if not same as hostname)", "hostname" },
     { "version", 0, arg_flag, &version_flag },
     { "help", 0, arg_flag, &help_flag }
 };
@@ -654,6 +746,8 @@ main(int argc, char **argv)
 	exit(0);
     }
 
+    setup_signal();
+
     if (config_file == NULL)
 	config_file = HDB_DB_DIR "/kdc.conf";
 
@@ -711,9 +805,12 @@ main(int argc, char **argv)
     signal_fd = make_signal_socket (context);
     listen_fd = make_listen_socket (context, port_str);
 
-    signal (SIGPIPE, SIG_IGN);
+    kadm5_log_get_version_fd (log_fd, &current_version);
 
-    for (;;) {
+    krb5_warnx(context, "ipropd-master started at version: %lu", 
+	       (unsigned long)current_version);
+
+    while(exit_flag == 0){
 	slave *p;
 	fd_set readset;
 	int max_fd = 0;
@@ -750,6 +847,10 @@ main(int argc, char **argv)
 	    kadm5_log_get_version_fd (log_fd, &current_version);
 
 	    if (current_version > old_version) {
+		krb5_warnx(context, 
+			   "Missed a signal, updating slaves %lu to %lu",
+			   (unsigned long)old_version,
+			   (unsigned long)current_version);
 		for (p = slaves; p != NULL; p = p->next) {
 		    if (p->flags & SLAVE_F_DEAD)
 			continue;
@@ -771,8 +872,18 @@ main(int argc, char **argv)
 	    assert(ret >= 0);
 	    old_version = current_version;
 	    kadm5_log_get_version_fd (log_fd, &current_version);
-	    for (p = slaves; p != NULL; p = p->next)
-		send_diffs (context, p, log_fd, database, current_version);
+	    if (current_version > old_version) {
+		krb5_warnx(context, 
+			   "Got a signal, updating slaves %lu to %lu",
+			   (unsigned long)old_version,
+			   (unsigned long)current_version);
+		for (p = slaves; p != NULL; p = p->next)
+		    send_diffs (context, p, log_fd, database, current_version);
+	    } else {
+		krb5_warnx(context, 
+			   "Got a signal, but no update in log version %lu",
+			   (unsigned long)current_version);
+	    }
         }
 
 	for(p = slaves; p != NULL; p = p->next) {
@@ -782,11 +893,13 @@ main(int argc, char **argv)
 		--ret;
 		assert(ret >= 0);
 		if(process_msg (context, p, log_fd, database, current_version))
-		    slave_dead(p);
+		    slave_dead(context, p);
 	    } else if (slave_gone_p (p))
-		slave_dead (p);
-	    else if (slave_missing_p (p))
+		slave_dead(context, p);
+	    else if (slave_missing_p (p)) {
+		krb5_warnx(context, "slave %s missing, sending AYT", p->name);
 		send_are_you_there (context, p);
+	    }
 	}
 
 	if (ret && FD_ISSET(listen_fd, &readset)) {
@@ -797,5 +910,15 @@ main(int argc, char **argv)
 	write_stats(context, slaves, current_version);
     }
 
+    if(exit_flag == SIGXCPU)
+	krb5_warnx(context, "%s CPU time limit exceeded", getprogname());
+    else if(exit_flag == SIGINT || exit_flag == SIGTERM)
+	krb5_warnx(context, "%s terminated", getprogname());
+    else
+	krb5_warnx(context, "%s unexpected exit reason: %d", 
+		   getprogname(), exit_flag);
+
+    write_master_down(context);
+
     return 0;
 }
