@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 - 2005 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2007 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -33,11 +33,12 @@
 
 #include "iprop.h"
 
-RCSID("$Id: ipropd_slave.c 19078 2006-11-20 18:12:41Z lha $");
+RCSID("$Id: ipropd_slave.c 21818 2007-08-03 14:18:04Z lha $");
 
 static krb5_log_facility *log_facility;
 static char *server_time_lost = "5 min";
 static int time_before_lost;
+const char *slave_str = NULL;
 
 static int
 connect_to_master (krb5_context context, const char *master,
@@ -80,7 +81,7 @@ connect_to_master (krb5_context context, const char *master,
 
 static void
 get_creds(krb5_context context, const char *keytab_str,
-	  krb5_ccache *cache, const char *host)
+	  krb5_ccache *cache, const char *serverhost)
 {
     krb5_keytab keytab;
     krb5_principal client;
@@ -101,14 +102,15 @@ get_creds(krb5_context context, const char *keytab_str,
     if(ret)
 	krb5_err(context, 1, ret, "%s", keytab_str);
     
-    ret = krb5_sname_to_principal (context, NULL, IPROP_NAME,
+
+    ret = krb5_sname_to_principal (context, slave_str, IPROP_NAME,
 				   KRB5_NT_SRV_HST, &client);
     if (ret) krb5_err(context, 1, ret, "krb5_sname_to_principal");
 
     ret = krb5_get_init_creds_opt_alloc(context, &init_opts);
     if (ret) krb5_err(context, 1, ret, "krb5_get_init_creds_opt_alloc");
 
-    asprintf (&server, "%s/%s", IPROP_NAME, host);
+    asprintf (&server, "%s/%s", IPROP_NAME, serverhost);
     if (server == NULL)
 	krb5_errx (context, 1, "malloc: no memory");
 
@@ -160,8 +162,12 @@ receive_loop (krb5_context context,
     int ret;
     off_t left, right;
     void *buf;
-    int32_t vers;
+    int32_t vers, vers2;
+    ssize_t sret;
 
+    /*
+     * Seek to the current version of the local database.
+     */
     do {
 	int32_t len, timestamp, tmp;
 	enum kadm_ops op;
@@ -176,40 +182,92 @@ receive_loop (krb5_context context,
 	    krb5_storage_seek(sp, len + 8, SEEK_CUR);
     } while(vers <= server_context->log_context.version);
 
+    /*
+     * Read up rest of the entires into the memory...
+     */
     left  = krb5_storage_seek (sp, -16, SEEK_CUR);
     right = krb5_storage_seek (sp, 0, SEEK_END);
     buf = malloc (right - left);
-    if (buf == NULL && (right - left) != 0) {
-	krb5_warnx (context, "malloc: no memory");
-	return;
-    }
+    if (buf == NULL && (right - left) != 0)
+	krb5_errx (context, 1, "malloc: no memory");
+
+    /*
+     * ...and then write them out to the on-disk log.
+     */
     krb5_storage_seek (sp, left, SEEK_SET);
     krb5_storage_read (sp, buf, right - left);
-    write (server_context->log_context.log_fd, buf, right-left);
-    fsync (server_context->log_context.log_fd);
+    sret = write (server_context->log_context.log_fd, buf, right-left);
+    if (sret != right - left)
+	krb5_err(context, 1, errno, "Failed to write log to disk");
+    ret = fsync (server_context->log_context.log_fd);
+    if (ret)
+	krb5_err(context, 1, errno, "Failed to sync log to disk");
     free (buf);
 
+    /*
+     * Go back to the startpoint and start to commit the entires to
+     * the database.
+     */
     krb5_storage_seek (sp, left, SEEK_SET);
 
     for(;;) {
-	int32_t len, timestamp, tmp;
+	int32_t len, len2, timestamp, tmp;
+	off_t cur, cur2;
 	enum kadm_ops op;
 
 	if(krb5_ret_int32 (sp, &vers) != 0)
 	    break;
-	krb5_ret_int32 (sp, &timestamp);
-	krb5_ret_int32 (sp, &tmp);
+	ret = krb5_ret_int32 (sp, &timestamp);
+	if (ret) krb5_errx(context, 1, "entry %ld: too short", (long)vers);
+	ret = krb5_ret_int32 (sp, &tmp);
+	if (ret) krb5_errx(context, 1, "entry %ld: too short", (long)vers);
 	op = tmp;
-	krb5_ret_int32 (sp, &len);
+	ret = krb5_ret_int32 (sp, &len);
+	if (ret) krb5_errx(context, 1, "entry %ld: too short", (long)vers);
+	if (len < 0)
+	    krb5_errx(context, 1, "log is corrupted, "
+			"negative length of entry version %ld: %ld",
+			(long)vers, (long)len);
+	cur = krb5_storage_seek(sp, 0, SEEK_CUR);
+
+	krb5_warnx (context, "replaying entry %d", (int)vers);
 
 	ret = kadm5_log_replay (server_context,
 				op, vers, len, sp);
 	if (ret)
-	    krb5_warn (context, ret, "kadm5_log_replay: %d", (int)vers);
-	else
-	    server_context->log_context.version = vers;
-	krb5_storage_seek (sp, 8, SEEK_CUR);
+	    krb5_warn (context, ret,
+		       "kadm5_log_replay: %ld. Lost entry entry, "
+		       "Database out of sync ?",
+
+		       (long)vers);
+	{
+	    /* 
+	     * Make sure the krb5_log_replay does the right thing wrt
+	     * reading out data from the sp.
+	     */
+	    cur2 = krb5_storage_seek(sp, 0, SEEK_CUR);
+	    if (cur + len != cur2)
+		krb5_errx(context, 1, 
+			  "kadm5_log_reply version: %ld didn't read the whole entry",
+			  (long)vers);
+	}
+
+	if (krb5_ret_int32 (sp, &len2) != 0)
+	    krb5_errx(context, 1, "entry %ld: postamble too short", (long)vers);
+	if(krb5_ret_int32 (sp, &vers2) != 0)
+	    krb5_errx(context, 1, "entry %ld: postamble too short", (long)vers);
+
+	if (len != len2)
+	    krb5_errx(context, 1, "entry %ld: len != len2", (long)vers);
+	if (vers != vers2)
+	    krb5_errx(context, 1, "entry %ld: vers != vers2", (long)vers);
     }
+
+    /*
+     * Update version
+     */
+
+    server_context->log_context.version = vers;
 }
 
 static void
@@ -271,6 +329,8 @@ receive_everything (krb5_context context, int fd,
     char *dbname;
     HDB *mydb;
   
+    krb5_warnx(context, "receive complete database");
+
     asprintf(&dbname, "%s-NEW", server_context->db->hdb_name);
     ret = hdb_create(context, &mydb, dbname);
     if(ret)
@@ -285,7 +345,6 @@ receive_everything (krb5_context context, int fd,
     /* I really want to use O_EXCL here, but given that I can't easily clean
        up on error, I won't */
     ret = mydb->hdb_open(context, mydb, O_RDWR | O_CREAT | O_TRUNC, 0600);
-
     if (ret)
 	krb5_err (context, 1, ret, "db->open");
 
@@ -359,6 +418,8 @@ receive_everything (krb5_context context, int fd,
     ret = mydb->hdb_destroy (context, mydb);
     if (ret)
 	krb5_err (context, 1, ret, "db->destroy");
+
+    krb5_warnx(context, "receive complete database, version %ld", (long)vno);
 }
 
 static char *config_file;
@@ -380,6 +441,8 @@ static struct getargs args[] = {
       "port ipropd-slave will connect to", "port"},
     { "detach", 0, arg_flag, &detach_from_console, 
       "detach from console" },
+    { "hostname", 0, arg_string, &slave_str, 
+      "hostname of slave (if not same as hostname)", "hostname" },
     { "version", 0, arg_flag, &version_flag },
     { "help", 0, arg_flag, &help_flag }
 };
@@ -399,10 +462,11 @@ main(int argc, char **argv)
     krb5_ccache ccache;
     krb5_principal server;
     char **files;
+    int optidx;
 
     const char *master;
     
-    krb5_program_setup(&context, argc, argv, args, num_args, NULL);
+    optidx = krb5_program_setup(&context, argc, argv, args, num_args, NULL);
     
     if(help_flag)
 	krb5_std_usage(0, args, num_args);
@@ -411,6 +475,8 @@ main(int argc, char **argv)
 	exit(0);
     }
 
+    setup_signal();
+
     if (config_file == NULL)
 	config_file = HDB_DB_DIR "/kdc.conf";
 
@@ -423,8 +489,8 @@ main(int argc, char **argv)
     if (ret)
 	krb5_err(context, 1, ret, "reading configuration files");
 
-    argc -= optind;
-    argv += optind;
+    argc -= optidx;
+    argv += optidx;
 
     if (argc != 1)
 	krb5_std_usage(1, args, num_args);
@@ -482,10 +548,13 @@ main(int argc, char **argv)
     if (ret)
 	krb5_err (context, 1, ret, "krb5_sendauth");
 
+    krb5_warnx(context, "ipropd-slave started at version: %ld",
+	       (long)server_context->log_context.version);
+
     ihave (context, auth_context, master_fd,
 	   server_context->log_context.version);
 
-    for (;;) {
+    while (exit_flag == 0) {
 	krb5_data out;
 	krb5_storage *sp;
 	int32_t tmp;
@@ -545,5 +614,13 @@ main(int argc, char **argv)
 	krb5_data_free (&out);
     }
     
+    if(exit_flag == SIGXCPU)
+	krb5_warnx(context, "%s CPU time limit exceeded", getprogname());
+    else if(exit_flag == SIGINT || exit_flag == SIGTERM)
+	krb5_warnx(context, "%s terminated", getprogname());
+    else
+	krb5_warnx(context, "%s unexpected exit reason: %d", 
+		   getprogname(), exit_flag);
+
     return 0;
 }
