@@ -33,7 +33,7 @@
 
 #include "kadm5_locl.h"
 
-RCSID("$Id: log.c 17445 2006-05-05 10:37:46Z lha $");
+RCSID("$Id: log.c 21811 2007-08-03 14:04:32Z lha $");
 
 /*
  * A log record consists of:
@@ -124,6 +124,7 @@ kadm5_log_reinit (kadm5_server_context *context)
     kadm5_log_context *log_context = &context->log_context;
 
     if (log_context->log_fd != -1) {
+	flock (log_context->log_fd, LOCK_UN);
 	close (log_context->log_fd);
 	log_context->log_fd = -1;
     }
@@ -276,13 +277,18 @@ kadm5_log_replay_create (kadm5_server_context *context,
     memset(&ent, 0, sizeof(ent));
 
     ret = krb5_data_alloc (&data, len);
-    if (ret)
+    if (ret) {
+	krb5_set_error_string(context->context, "out of memory");
 	return ret;
+    }
     krb5_storage_read (sp, data.data, len);
     ret = hdb_value2entry (context->context, &data, &ent.entry);
     krb5_data_free(&data);
-    if (ret)
+    if (ret) {
+	krb5_set_error_string(context->context, 
+			      "Unmarshaling hdb entry failed");
 	return ret;
+    }
     ret = context->db->hdb_store(context->context, context->db, 0, &ent);
     hdb_free_entry (context->context, &ent);
     return ret;
@@ -668,6 +674,8 @@ kadm5_log_replay_modify (kadm5_server_context *context,
 
 	ent.entry.keys.len = num;
 	ent.entry.keys.val = malloc(len * sizeof(*ent.entry.keys.val));
+	if (ent.entry.keys.val == NULL)
+	    return ENOMEM;
 	for (i = 0; i < ent.entry.keys.len; ++i) {
 	    ret = copy_Key(&log_ent.entry.keys.val[i],
 			   &ent.entry.keys.val[i]);
@@ -678,13 +686,12 @@ kadm5_log_replay_modify (kadm5_server_context *context,
     if ((mask & KADM5_TL_DATA) && log_ent.entry.extensions) {
 	HDB_extensions *es = ent.entry.extensions;
 
-	if (ent.entry.extensions == NULL) {
-	    ent.entry.extensions = calloc(1, sizeof(*ent.entry.extensions));
-	    if (ent.entry.extensions == NULL)
-		goto out;
-	}
+	ent.entry.extensions = calloc(1, sizeof(*ent.entry.extensions));
+	if (ent.entry.extensions == NULL)
+	    goto out;
 
-	ret = copy_HDB_extensions(es, ent.entry.extensions);
+	ret = copy_HDB_extensions(log_ent.entry.extensions,
+				  ent.entry.extensions);
 	if (ret) {
 	    free(ent.entry.extensions);
 	    ent.entry.extensions = es;
@@ -704,7 +711,7 @@ kadm5_log_replay_modify (kadm5_server_context *context,
 }
 
 /*
- * Add a `nop' operation to the log.
+ * Add a `nop' operation to the log. Does not close the log.
  */
 
 kadm5_ret_t
@@ -729,9 +736,7 @@ kadm5_log_nop (kadm5_server_context *context)
     }
     ret = kadm5_log_flush (log_context, sp);
     krb5_storage_free (sp);
-    if (ret)
-	return ret;
-    ret = kadm5_log_end (context);
+
     return ret;
 }
 
@@ -769,7 +774,7 @@ kadm5_log_foreach (kadm5_server_context *context,
     lseek (fd, 0, SEEK_SET);
     sp = krb5_storage_from_fd (fd);
     for (;;) {
-	int32_t ver, timestamp, op, len;
+	int32_t ver, timestamp, op, len, len2, ver2;
 
 	if(krb5_ret_int32 (sp, &ver) != 0)
 	    break;
@@ -777,8 +782,14 @@ kadm5_log_foreach (kadm5_server_context *context,
 	krb5_ret_int32 (sp, &op);
 	krb5_ret_int32 (sp, &len);
 	(*func)(context, ver, timestamp, op, len, sp, ctx);
-	krb5_storage_seek(sp, 8, SEEK_CUR);
+	krb5_ret_int32 (sp, &len2);
+	krb5_ret_int32 (sp, &ver2);
+	if (len != len2)
+	    abort();
+	if (ver != ver2)
+	    abort();
     }
+    krb5_storage_free(sp);
     return 0;
 }
 
@@ -798,6 +809,10 @@ kadm5_log_goto_end (int fd)
 
 /*
  * Return previous log entry.
+ * 
+ * The pointer in `sp is assumed to be at the top of the entry before
+ * previous entry. On success, the `sp pointer is set to data portion
+ * of previous entry. In case of error, its not changed at all.
  */
 
 kadm5_ret_t
@@ -809,9 +824,11 @@ kadm5_log_previous (krb5_context context,
 		    uint32_t *len)
 {
     krb5_error_code ret;
-    off_t off;
+    off_t off, oldoff;
     int32_t tmp;
 
+    oldoff = krb5_storage_seek(sp, 0, SEEK_CUR);
+
     krb5_storage_seek(sp, -8, SEEK_CUR);
     ret = krb5_ret_int32 (sp, &tmp);
     if (ret)
@@ -825,6 +842,7 @@ kadm5_log_previous (krb5_context context,
     if (ret)
 	goto end_of_storage;
     if (tmp != *ver) {
+	krb5_storage_seek(sp, oldoff, SEEK_SET);
 	krb5_set_error_string(context, "kadm5_log_previous: log entry "
 			      "have consistency failure, version number wrong");
 	return KADM5_BAD_DB;
@@ -838,7 +856,8 @@ kadm5_log_previous (krb5_context context,
     ret = krb5_ret_int32 (sp, &tmp);
     if (ret)
 	goto end_of_storage;
-    if (tmp != *ver) {
+    if (tmp != *len) {
+	krb5_storage_seek(sp, oldoff, SEEK_SET);
 	krb5_set_error_string(context, "kadm5_log_previous: log entry "
 			      "have consistency failure, length wrong");
 	return KADM5_BAD_DB;
@@ -846,6 +865,7 @@ kadm5_log_previous (krb5_context context,
     return 0;
 
  end_of_storage:
+    krb5_storage_seek(sp, oldoff, SEEK_SET);
     krb5_set_error_string(context, "kadm5_log_previous: end of storage "
 			  "reached before end");
     return ret;
@@ -900,7 +920,7 @@ kadm5_log_truncate (kadm5_server_context *server_context)
     if (ret)
 	return ret;
 
-    ret = kadm5_log_set_version (server_context, vno + 1);
+    ret = kadm5_log_set_version (server_context, vno);
     if (ret)
 	return ret;
 
@@ -914,3 +934,14 @@ kadm5_log_truncate (kadm5_server_context *server_context)
     return 0;
 
 }
+
+const char *
+kadm5_log_signal_socket(krb5_context context)
+{
+    return krb5_config_get_string_default(context,
+					  NULL,
+					  KADM5_LOG_SIGNAL,
+					  "kdc",
+					  "signal_socket",
+					  NULL);
+}
