@@ -33,7 +33,7 @@
 
 #include "kadm5_locl.h"
 
-RCSID("$Id: rename_s.c 17579 2006-05-13 21:22:55Z lha $");
+RCSID("$Id: rename_s.c 21811 2007-08-03 14:04:32Z lha $");
 
 kadm5_ret_t
 kadm5_s_rename_principal(void *server_handle, 
@@ -72,6 +72,8 @@ kadm5_s_rename_principal(void *server_handle,
 	    if(ent.entry.keys.val[i].salt == NULL){
 		ent.entry.keys.val[i].salt = 
 		    malloc(sizeof(*ent.entry.keys.val[i].salt));
+		if(ent.entry.keys.val[i].salt == NULL)
+		    return ENOMEM;
 		ret = copy_Salt(&salt, ent.entry.keys.val[i].salt);
 		if(ret)
 		    break;
@@ -90,9 +92,7 @@ kadm5_s_rename_principal(void *server_handle,
 	goto out2;
     }
 
-    kadm5_log_rename (context,
-		      source,
-		      &ent.entry);
+    kadm5_log_rename (context, source, &ent.entry);
 
     ret = context->db->hdb_store(context->context, context->db, 0, &ent);
     if(ret){
