@@ -43,7 +43,7 @@
 
 #include "kcm.h"
 
-RCSID("$Id: kcm.c,v 1.6 2005/05/29 14:49:50 lha Exp $");
+RCSID("$Id: kcm.c 17442 2006-05-05 09:31:15Z lha $");
 
 typedef struct krb5_kcmcache {
     char *name;
@@ -53,7 +53,7 @@ typedef struct krb5_kcmcache {
 
 #define KCMCACHE(X)	((krb5_kcmcache *)(X)->data.data)
 #define CACHENAME(X)	(KCMCACHE(X)->name)
-#define KCMCURSOR(C)	(*(u_int32_t *)(C))
+#define KCMCURSOR(C)	(*(uint32_t *)(C))
 
 static krb5_error_code
 try_door(krb5_context context, const krb5_kcmcache *k,
@@ -106,7 +106,7 @@ try_unix_socket(krb5_context context, const krb5_kcmcache *k,
     if (fd < 0)
 	return KRB5_CC_IO;
     
-    if (connect(fd, (struct sockaddr *)&k->path, sizeof(k->path)) != 0) {
+    if (connect(fd, rk_UNCONST(&k->path), sizeof(k->path)) != 0) {
 	close(fd);
 	return KRB5_CC_IO;
     }
@@ -246,7 +246,8 @@ kcm_call(krb5_context context,
 	 krb5_data *response_data_p)
 {
     krb5_data response_data;
-    krb5_error_code ret, status;
+    krb5_error_code ret;
+    int32_t status;
     krb5_storage *response;
 
     if (response_p != NULL)
@@ -521,7 +522,7 @@ kcm_retrieve(krb5_context context,
 	return ret;
     }
 
-    ret = krb5_store_creds_tag(request, (krb5_creds *)mcred);
+    ret = krb5_store_creds_tag(request, rk_UNCONST(mcred));
     if (ret) {
 	krb5_storage_free(request);
 	return ret;
@@ -605,7 +606,7 @@ kcm_get_first (krb5_context context,
     krb5_kcmcache *k = KCMCACHE(id);
     krb5_storage *request, *response;
     krb5_data response_data;
-    u_int32_t tmp;
+    int32_t tmp;
 
     ret = kcm_storage_request(context, KCM_OP_GET_FIRST, &request);
     if (ret)
@@ -624,7 +625,7 @@ kcm_get_first (krb5_context context,
     }
 
     ret = krb5_ret_int32(response, &tmp);
-    if (ret)
+    if (ret || tmp < 0)
 	ret = KRB5_CC_IO;
 
     krb5_storage_free(request);
@@ -902,7 +903,7 @@ _krb5_kcm_noop(krb5_context context,
 krb5_error_code
 _krb5_kcm_chmod(krb5_context context,
 		krb5_ccache id,
-		u_int16_t mode)
+		uint16_t mode)
 {
     krb5_error_code ret;
     krb5_kcmcache *k = KCMCACHE(id);
@@ -943,8 +944,8 @@ _krb5_kcm_chmod(krb5_context context,
 krb5_error_code
 _krb5_kcm_chown(krb5_context context,
 		krb5_ccache id,
-		u_int32_t uid,
-		u_int32_t gid)
+		uint32_t uid,
+		uint32_t gid)
 {
     krb5_error_code ret;
     krb5_kcmcache *k = KCMCACHE(id);
