@@ -32,7 +32,7 @@
 
 #include "krb5_locl.h"
 
-RCSID("$Id: n-fold-test.c,v 1.5 2004/05/25 21:35:12 lha Exp $");
+RCSID("$Id: n-fold-test.c 16682 2006-02-01 12:45:13Z lha $");
 
 enum { MAXSIZE = 24 };
 
@@ -92,7 +92,7 @@ static struct testcase {
     {NULL, 0}
 };
 
-int KRB5_LIB_FUNCTION
+int
 main(int argc, char **argv)
 {
     unsigned char data[MAXSIZE];
