@@ -35,7 +35,7 @@
 #include <getarg.h>
 #include <parse_bytes.h>
 #include <err.h>
-RCSID("$Id: verify_krb5_conf.c,v 1.33 2005/06/14 00:06:17 lha Exp $");
+RCSID("$Id: verify_krb5_conf.c 19194 2006-11-29 19:19:45Z lha $");
 
 /* verify krb5.conf */
 
@@ -368,11 +368,13 @@ struct entry libdefaults_entries[] = {
     { "capath", krb5_config_list, all_strings },
     { "clockskew", krb5_config_string, check_time },
     { "date_format", krb5_config_string, NULL },
+    { "default_cc_name", krb5_config_string, NULL },
     { "default_etypes", krb5_config_string, NULL },
     { "default_etypes_des", krb5_config_string, NULL },
     { "default_keytab_modify_name", krb5_config_string, NULL },
     { "default_keytab_name", krb5_config_string, NULL },
     { "default_realm", krb5_config_string, NULL },
+    { "dns_canonize_hostname", krb5_config_string, check_boolean },
     { "dns_proxy", krb5_config_string, NULL },
     { "dns_lookup_kdc", krb5_config_string, check_boolean },
     { "dns_lookup_realm", krb5_config_string, check_boolean },
@@ -400,7 +402,6 @@ struct entry libdefaults_entries[] = {
     { "v4_instance_resolve", krb5_config_string, check_boolean },
     { "v4_name_convert", krb5_config_list, v4_name_convert_entries },
     { "verify_ap_req_nofail", krb5_config_string, check_boolean },
-    { "pkinit-openssl-engine", krb5_config_string, NULL },
     { "max_retries", krb5_config_string, check_time },
     { "renew_lifetime", krb5_config_string, check_time },
     { "proxiable", krb5_config_string, check_boolean },
@@ -423,7 +424,12 @@ struct entry appdefaults_entries[] = {
     { "renew_lifetime", krb5_config_string, check_time },
     { "no-addresses", krb5_config_string, check_boolean },
     { "krb4_get_tickets", krb5_config_string, check_boolean },
-    { "pkinit-anchors", krb5_config_string, NULL },
+    { "pkinit_anchors", krb5_config_string, NULL },
+    { "pkinit_win2k", krb5_config_string, NULL },
+    { "pkinit_win2k_require_binding", krb5_config_string, NULL },
+    { "pkinit_require_eku", krb5_config_string, NULL },
+    { "pkinit_require_krbtgt_otherName", krb5_config_string, NULL },
+    { "pkinit_require_hostname_match", krb5_config_string, NULL },
 #if 0
     { "anonymous", krb5_config_string, check_boolean },
 #endif
@@ -499,8 +505,14 @@ struct entry kdc_entries[] = {
     { "kdc_warn_pwexpire", krb5_config_string, check_time },
     { "use_2b", krb5_config_list, NULL },
     { "enable-pkinit", krb5_config_string, check_boolean },
-    { "pki-identity", krb5_config_string, NULL },
-    { "pki-anchors", krb5_config_string, NULL },
+    { "pkinit_identity", krb5_config_string, NULL },
+    { "pkinit_anchors", krb5_config_string, NULL },
+    { "pkinit_pool", krb5_config_string, NULL },
+    { "pkinit_revoke", krb5_config_string, NULL },
+    { "pkinit_kdc_ocsp", krb5_config_string, NULL },
+    { "pkinit_principal_in_certificate", krb5_config_string, NULL },
+    { "pkinit_dh_min_bits", krb5_config_string, NULL },
+    { "pkinit_allow_proxy_certificate", krb5_config_string, NULL },
     { "hdb-ldap-create-base", krb5_config_string, NULL },
     { "v4-realm", krb5_config_string, NULL },
     { NULL }
@@ -534,6 +546,8 @@ struct entry capaths_entries[] = {
 struct entry password_quality_entries[] = {
     { "policies", krb5_config_string, NULL },
     { "external_program", krb5_config_string, NULL },
+    { "min_classes", krb5_config_string, check_numeric },
+    { "min_length", krb5_config_string, check_numeric },
     { "", krb5_config_list, all_strings },
     { NULL }
 };
@@ -619,7 +633,7 @@ main(int argc, char **argv)
     krb5_context context;
     krb5_error_code ret;
     krb5_config_section *tmp_cf;
-    int optind = 0;
+    int optidx = 0;
 
     setprogname (argv[0]);
 
@@ -629,7 +643,7 @@ main(int argc, char **argv)
     else if (ret)
 	errx (1, "krb5_init_context failed with %d", ret);
 
-    if(getarg(args, sizeof(args) / sizeof(args[0]), argc, argv, &optind))
+    if(getarg(args, sizeof(args) / sizeof(args[0]), argc, argv, &optidx))
 	usage(1);
     
     if (help_flag)
@@ -640,8 +654,8 @@ main(int argc, char **argv)
 	exit(0);
     }
 
-    argc -= optind;
-    argv += optind;
+    argc -= optidx;
+    argv += optidx;
 
     tmp_cf = NULL;
     if(argc == 0)
