@@ -36,7 +36,7 @@
 #include <config.h>
 #endif
 #include "roken.h"
-RCSID("$Id: getcap.c,v 1.10 2005/04/12 11:28:44 lha Exp $");
+RCSID("$Id: getcap.c 16561 2006-01-13 14:25:32Z lha $");
 
 #include <sys/types.h>
 #include <ctype.h>
@@ -69,9 +69,14 @@ static size_t	 topreclen;	/* toprec length */
 static char	*toprec;	/* Additional record specified by cgetset() */
 static int	 gottoprec;	/* Flag indicating retrieval of toprecord */
 
+#if 0 /*
+       * Don't use db support unless its build into libc but we dont
+       * check for that now, so just disable the code.
+       */
 #if defined(HAVE_DBOPEN) && defined(HAVE_DB_H)
 #define USE_DB
 #endif
+#endif
 
 #ifdef USE_DB
 static int	cdbget (DB *, char **, const char *);
