@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 - 2002 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2000 - 2002, 2005 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: getifaddrs.c,v 1.11 2005/04/30 15:45:47 lha Exp $");
+RCSID("$Id: getifaddrs.c 19310 2006-12-11 18:59:31Z lha $");
 #endif
 #include "roken.h"
 
@@ -56,6 +56,21 @@ struct mbuf;
 
 #include <ifaddrs.h>
 
+#ifdef __hpux
+#define lifconf if_laddrconf
+#define lifc_len iflc_len
+#define lifc_buf iflc_buf
+#define lifc_req iflc_req
+
+#define lifreq if_laddrreq
+#define lifr_addr iflr_addr
+#define lifr_name iflr_name
+#define lifr_dstaddr iflr_dstaddr
+#define lifr_broadaddr iflr_broadaddr
+#define lifr_flags iflr_flags
+#define lifr_index iflr_index
+#endif
+
 #ifdef AF_NETLINK
 
 /*
@@ -108,6 +123,7 @@ struct mbuf;
 #include <linux/rtnetlink.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/poll.h>
 #include <netpacket/packet.h>
 #include <net/ethernet.h>     /* the L2 protocols */
 #include <sys/uio.h>
@@ -378,13 +394,30 @@ nl_getlist(int sd, int seq,
   struct nlmsghdr *nlh = NULL;
   int status;
   int done = 0;
+  int tries = 3;
 
+ try_again:
   status = nl_sendreq(sd, request, NLM_F_ROOT|NLM_F_MATCH, &seq);
   if (status < 0)
     return status;
   if (seq == 0)
     seq = (int)time(NULL);
   while(!done){
+    struct pollfd pfd;
+
+    pfd.fd = sd;
+    pfd.events = POLLIN | POLLPRI;
+    pfd.revents = 0;
+    status = poll(&pfd, 1, 1000);
+    if (status < 0)
+	return status;
+    else if (status == 0) {
+	seq++;
+	if (tries-- > 0)
+	    goto try_again;
+	return -1;
+    }
+
     status = nl_getmsg(sd, request, seq, &nlh, &done);
     if (status < 0)
       return status;
@@ -417,16 +450,17 @@ nl_getlist(int sd, int seq,
 static void 
 free_nlmsglist(struct nlmsg_list *nlm0)
 {
-  struct nlmsg_list *nlm;
+  struct nlmsg_list *nlm, *nlm_next;
   int saved_errno;
   if (!nlm0)
     return;
   saved_errno = errno;
-  for (nlm=nlm0; nlm; nlm=nlm->nlm_next){
+  for (nlm=nlm0; nlm; nlm=nlm_next){
     if (nlm->nlh)
       free(nlm->nlh);
+    nlm_next=nlm->nlm_next;
+    free(nlm);
   }
-  free(nlm0);
   __set_errno(saved_errno);
 }
 
@@ -468,7 +502,7 @@ nl_open(void)
 
 /* ====================================================================== */
 int ROKEN_LIB_FUNCTION
-getifaddrs(struct ifaddrs **ifap)
+rk_getifaddrs(struct ifaddrs **ifap)
 {
   int sd;
   struct nlmsg_list *nlmsg_list, *nlmsg_end, *nlm;
@@ -824,7 +858,7 @@ getifaddrs(struct ifaddrs **ifap)
 
 /* ---------------------------------------------------------------------- */
 void ROKEN_LIB_FUNCTION
-freeifaddrs(struct ifaddrs *ifa)
+rk_freeifaddrs(struct ifaddrs *ifa)
 {
   free(ifa);
 }
@@ -954,7 +988,7 @@ getifaddrs2(struct ifaddrs **ifap,
     free(buf);
     return 0;
   error_out:
-    freeifaddrs(start);
+    rk_freeifaddrs(start);
     close(fd);
     free(buf);
     errno = ret;
@@ -992,8 +1026,10 @@ getlifaddrs2(struct ifaddrs **ifap,
 	    ret = ENOMEM;
 	    goto error_out;
 	}
+#ifndef __hpux
 	ifconf.lifc_family = AF_UNSPEC;
 	ifconf.lifc_flags  = 0;
+#endif
 	ifconf.lifc_len    = buf_size;
 	ifconf.lifc_buf    = buf;
 
@@ -1078,7 +1114,7 @@ getlifaddrs2(struct ifaddrs **ifap,
     free(buf);
     return 0;
   error_out:
-    freeifaddrs(start);
+    rk_freeifaddrs(start);
     close(fd);
     free(buf);
     errno = ret;
@@ -1087,7 +1123,7 @@ getlifaddrs2(struct ifaddrs **ifap,
 #endif /* defined(HAVE_IPV6) && defined(SIOCGLIFCONF) && defined(SIOCGLIFFLAGS) */
 
 int ROKEN_LIB_FUNCTION
-getifaddrs(struct ifaddrs **ifap) 
+rk_getifaddrs(struct ifaddrs **ifap) 
 {
     int ret = -1;
     errno = ENXIO;
@@ -1115,7 +1151,7 @@ getifaddrs(struct ifaddrs **ifap)
 }
 
 void ROKEN_LIB_FUNCTION
-freeifaddrs(struct ifaddrs *ifp)
+rk_freeifaddrs(struct ifaddrs *ifp)
 {
     struct ifaddrs *p, *q;
     
