@@ -40,11 +40,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <err.h>
-#include <roken.h>
+#include "roken.h"
 
 #include "test-mem.h"
 
-RCSID("$Id: test-mem.c 15287 2005-05-29 21:21:12Z lha $");
+RCSID("$Id: test-mem.c 21005 2007-06-08 01:54:35Z lha $");
 
 /* #undef HAVE_MMAP */
 
